'use strict';

var path = require('path');
var fs = require('fs');
var prettier = require('prettier');
var _ = require('lodash');

function _interopDefaultLegacy (e) { return e && typeof e === 'object' && 'default' in e ? e : { 'default': e }; }

function _interopNamespace(e) {
  if (e && e.__esModule) return e;
  var n = Object.create(null);
  if (e) {
    Object.keys(e).forEach(function (k) {
      if (k !== 'default') {
        var d = Object.getOwnPropertyDescriptor(e, k);
        Object.defineProperty(n, k, d.get ? d : {
          enumerable: true,
          get: function () { return e[k]; }
        });
      }
    });
  }
  n["default"] = e;
  return Object.freeze(n);
}

var path__default = /*#__PURE__*/_interopDefaultLegacy(path);
var fs__default = /*#__PURE__*/_interopDefaultLegacy(fs);
var prettier__default = /*#__PURE__*/_interopDefaultLegacy(prettier);
var ___namespace = /*#__PURE__*/_interopNamespace(_);

const formatOption = {
  semi: false,
  singleQuote: true,
  trailingComma: "none",
  parser: "typescript"
};
const isThere = (p) => {
};
const readFile = (p) => {
  return fs__default["default"].readFileSync(p, "utf-8");
};
const mkdirsSync = (dirname) => {
  if (fs__default["default"].existsSync(dirname)) {
    return true;
  }
  if (mkdirsSync(path__default["default"].dirname(dirname))) {
    fs__default["default"].mkdirSync(dirname);
    return true;
  }
};
const create = (data, baseDir = "./") => {
  const { code, path: _path, children } = data;
  mkdirsSync(baseDir);
  const result = prettier__default["default"].format(code, formatOption);
  fs__default["default"].writeFileSync(path__default["default"].join(baseDir, _path), result);
  children.forEach((it) => create(it, baseDir));
};
var io = { isThere, readFile, create };

const matchCouple = (string) => {
  let data = string;
  return (start, end) => {
    if (!start || !end) {
      throw new Error("\u4E0D\u5177\u6709\u5339\u914D\u6761\u4EF6");
    }
    let index = string.indexOf(start);
    let count = 0;
    const before = ~index ? string.slice(0, index) : null;
    data = string.substring(index);
    index = 0;
    let stop = false;
    for (let c of data) {
      if (stop) {
        continue;
      }
      index++;
      if (c === start) {
        count++;
      }
      if (c === end) {
        count--;
      }
      if (!count) {
        stop = true;
      }
    }
    const main = data.slice(1, index - 1);
    const after = count ? null : data.slice(index);
    return [before, main, after];
  };
};

const complierComponent = ({
  template,
  imports,
  data,
  init,
  id,
  link,
  components,
  methods
}) => {
  const complierComponentImport = (list) => {
    return Object.entries(
      list.flat(Infinity).filter((it) => it).reduce((total, { target, from }) => {
        total[from] = (total[from] ? [...total[from], target] : [target]).flat(Infinity).filter((it) => !["module", "component"].includes(it.type));
        return total;
      }, {})
    ).filter(([, item]) => !___namespace.isEmpty(item)).map(([from, item]) => ({
      from,
      target: Array.from(
        new Set(item.map((it) => ___namespace.isString(it) ? it : it.name))
      )
    })).reduce(
      (total, { from, target }) => total + `import {${target.join(",")}} from '${from}'
`,
      ""
    );
  };
  const complierMethod = (list) => {
    return list.flat(Infinity).filter((it) => it).join("\n");
  };
  const complierInject = (list = []) => {
    return ___namespace.unionBy(
      list.flat(Infinity).filter((it) => it).map(({ target }) => target).flat(Infinity).filter(({ inject }) => inject),
      "name"
    ).map(({ name, inject }) => `public ${inject.name}: ${name}`).join(",");
  };
  const complierRegisterData = (list = []) => {
    return list.flat(Infinity).filter((it) => it).map(({ name }) => `${name}`).join("\n");
  };
  const complierInitData = (list = []) => {
    return list.flat(Infinity).filter((it) => it).map(
      ({ type, name, init: init2 }) => `this.${name} = ${init2 === "" ? '""' : init2}`
    ).join("\n");
  };
  const complierInit = (list = []) => {
    return list.flat(Infinity).filter((it) => it).map((it) => {
      if (___namespace.isFunction(it)) {
        const [, body] = matchCouple(it.toString())("{", "}");
        return body;
      }
      return it;
    }).join("\n");
  };
  const childrenComponent = components.flat(Infinity).filter((it) => it && ___namespace.isObject(it));
  const childrenImports = childrenComponent.map((it) => it.imports).flat(Infinity);
  return {
    code: `
    ${complierComponentImport(imports.concat(childrenImports))}
  @Component({
      selector: 'eo-${___namespace.kebabCase(id)}',
      template: \`${template}\`
  })
  export class ${id}Component implements OnInit {
    ${complierRegisterData(data)}
    constructor(${complierInject(imports)}) {
      ${complierInitData(data)}
    }
    async ngOnInit(): Promise<void> {
      ${complierInit(init)}
    }
   ${complierMethod(methods)}
  }

  ${childrenComponent.length ? childrenComponent.map((child) => {
      return `
      @Component({
          selector: 'child-${___namespace.kebabCase(child.id)}',
          template: \`${child.template}\`
      })
      export class ${child.id}Component implements OnInit {
        ${complierRegisterData(child.data)}
        constructor(${complierInject(child.imports.flat(Infinity))}) {
          ${complierInitData(child.data)}
        }
        async ngOnInit(): Promise<void> {
          ${complierInit(child.init)}
        }
      ${complierMethod(child.methods)}
      }
    `;
    }).join("\n") : ""}
  `,
    path: `./${___namespace.kebabCase(id)}.component.ts`,
    children: []
  };
};

const complierModule = ({
  id,
  children: _children,
  link,
  components,
  imports
}) => {
  const complierModuleImports = (list) => Object.entries(
    list.flat(Infinity).filter((it) => it).reduce((total, { target, from }) => {
      total[from] = (total[from] ? [...total[from], target] : [target]).flat(Infinity).filter(
        (it) => it.type && ["module", "component", "service", "base"].includes(it.type)
      );
      return total;
    }, {})
  ).map(([from, item]) => ({
    from,
    target: Array.from(new Set(item.map((it) => it.name)))
  })).filter(({ from, target }) => target.length).reduce(
    (total, { from, target }) => total + `import {${target.join(",")}} from '${from}'
`,
    ""
  );
  const moduleList = (list) => Array.from(
    new Set(
      list.flat(Infinity).filter((it) => it).reduce((total, { target }) => [...total, target], []).flat(Infinity).filter((it) => it.type === "module").map((it) => it.name)
    )
  );
  const serviceList = (list) => Array.from(
    new Set(
      list.flat(Infinity).filter((it) => it).reduce((total, { target }) => [...total, target], []).flat(Infinity).filter((it) => it.type === "service" && it.ignore !== true).map((it) => it.name)
    )
  );
  const componentList = (list) => Array.from(
    new Set(
      list.flat(Infinity).filter((it) => it).map((it) => it + "Component")
    )
  );
  const linkComponentList = (list) => Array.from(
    new Set(
      list.flat(Infinity).filter((it) => it).reduce((total, { target }) => [...total, target], []).flat(Infinity).filter((it) => it.type === "component").map((it) => it.name)
    )
  );
  const complierImportComponent = (list) => Array.from(new Set(list.flat(Infinity).filter((it) => it))).reduce(
    (total, name) => total + `import {${___namespace.upperFirst(name)}Component} from './${___namespace.kebabCase(
      name
    )}.component'
`,
    ""
  );
  const exportList = (list) => Array.from(
    new Set(
      list.flat(Infinity).filter((it) => it).reduce((total, { target }) => [...total, target], []).flat(Infinity).filter((it) => it.export).map((it) => it.name)
    )
  );
  const children = _children?.map((it) => it.render()).map(complierComponent) || [];
  return {
    code: `
    ${complierModuleImports(imports)}
    import {${id}RoutingModule} from './${___namespace.kebabCase(id)}-routing.module'
    ${complierImportComponent(components)}
    
    @NgModule({
      imports: [${id}RoutingModule,${moduleList(imports)}],
      declarations: [${componentList(components)}, ${linkComponentList(
      imports
    )}],
      exports: [${exportList(imports)}],
      providers: [${serviceList(imports)}],
    })
    export class ${id}Module {}
    `,
    path: `./${___namespace.kebabCase(id)}.module.ts`,
    children
  };
};

const complierService = () => {
};
const complierType = {
  component: complierComponent,
  module: complierModule,
  service: complierService
};
const complier = (c) => {
  const { type, ...data } = c.render();
  return complierType[type](data);
};
var index = {
  complier,
  io
};

module.exports = index;
//# sourceMappingURL=complier.js.map
