declare type importsType = {
    target: string | string[] | {
        name: string;
        type: 'module' | 'component' | 'service' | 'directive';
        inject: {
            name: string;
            export?: boolean;
        };
    }[];
    from: string;
    effect?: string;
    ignore?: boolean;
}[];
declare type elementStruct = {
    type: 'element';
    template: string;
    data: string[];
    methods: string[];
    imports: importsType;
    components?: any[];
    init?: any[];
    filters?: string[];
    watch?: string[];
    computed?: string[];
} | {
    type: 'component';
    id: string;
    template: string;
    link: {
        name: string;
        type: 'scss';
    }[];
    imports: importsType;
    data: {
        name: string;
        init: any;
        type?: any;
    }[];
    init: any[];
    methods: any[];
    components: any[];
} | {
    type: 'module';
    id: string;
    imports: importsType;
    link: {
        name: string;
        type: 'component';
    }[];
    components: any[];
    children: any[];
} | {
    type: 'service';
    imports: importsType;
};

declare class Render {
    elementType: any;
    children: any;
    eventCb: any;
    methods: any;
    status: any;
    attr: any;
    constructor({ children, attr, event, status, elementType }: {
        children: any;
        attr?: {};
        event?: {};
        status?: {};
        elementType?: string;
    });
    renderAttr(list: any): string[];
    renderEvent(event: any): any[];
    renderStatus(status: any): any[];
    renderChild(children: any): {
        template: string;
        imports: any[];
        data: any[];
        init: any[];
        components: any[];
        methods: any[];
        resetFn: any[];
        elementType: any[];
    };
    static toCamel(id: any): string;
    static callbackRender(data: any): any;
}

declare class Component extends Render {
    id: any;
    prefix: any;
    init: any;
    imports: any;
    link: any;
    constructor({ id, prefix, link, imports, children, init }: {
        id?: string;
        prefix?: string;
        link?: boolean;
        imports: any;
        children: any;
        init: any;
    });
    render(): elementStruct;
}

declare class Module extends Render {
    id: any;
    link: any;
    childrens: any;
    constructor({ id, children }: {
        id: any;
        children: any;
    });
    render(): elementStruct;
}

declare class Service {
    id: any;
    type: any;
    constructor({ id }: {
        id: any;
    });
    render(): elementStruct;
}

declare type elementType = {};
declare type initType = {
    tag: string;
    children: any[];
    class: (string | object)[];
    content: string;
};
declare class Element extends Render implements elementType {
    tag: any;
    class: any;
    content: any;
    constructor({ tag, children, content, class: cls }: initType);
    render(): elementStruct;
}

declare const _default: {};

export { Component, Element, Module, Render, Service, _default as default };
