import * as _ from "lodash";
import { matchCouple } from "./utils";

export const complierComponent = ({
  template,
  imports,
  data,
  init,
  id,
  link,
  components,
  methods
}) => {
  const complierComponentImport = (list) => {
    return Object.entries(
      list
        .flat(Infinity)
        .filter((it) => it)
        .reduce((total, { target, from }) => {
          total[from] = (total[from] ? [...total[from], target] : [target])
            .flat(Infinity)
            .filter((it) => !["module", "component"].includes(it.type));
          return total;
        }, {})
    )
      .filter(([, item]: any) => !_.isEmpty(item))
      .map(([from, item]: any) => ({
        from,
        target: Array.from(
          new Set(item.map((it) => (_.isString(it) ? it : it.name)))
        )
      }))
      .reduce(
        (total, { from, target }) =>
          total + `import {${target.join(",")}} from '${from}'\n`,
        ""
      );
  };

  const complierMethod = (list) => {
    return list
      .flat(Infinity)
      .filter((it) => it)
      .join("\n");
  };

  const complierInject = (list = []) => {
    // [{"name": "NzModalService","type": "service","inject": {"name": "modal"}}]
    return _.unionBy(
      list
        .flat(Infinity)
        .filter((it) => it)
        .map(({ target }) => target)
        .flat(Infinity)
        .filter(({ inject }) => inject),
      "name"
    )
      .map(({ name, inject }: any) => `public ${inject.name}: ${name}`)
      .join(",");
  };

  const complierRegisterData = (list = []) => {
    // [ { name: 'isVisible', init: false, type: ['boolean'] }]
    return list
      .flat(Infinity)
      .filter((it) => it)
      .map(({ name }) => `${name}`)
      .join("\n");
  };

  const complierInitData = (list = []) => {
    // [ { name: 'isVisible', init: false, type: ['boolean'] }]
    return list
      .flat(Infinity)
      .filter((it) => it)
      .map(
        ({ type, name, init }) => `this.${name} = ${init === "" ? '""' : init}`
      )
      .join("\n");
  };

  const complierInit = (list = []) => {
    return list
      .flat(Infinity)
      .filter((it) => it)
      .map((it: any) => {
        if (_.isFunction(it)) {
          const [, body] = matchCouple(it.toString())("{", "}");
          return body;
        }
        return it;
      })
      .join("\n");
  };
  const childrenComponent = components
    .flat(Infinity)
    .filter((it) => it && _.isObject(it));
  const childrenImports = childrenComponent
    .map((it) => it.imports)
    .flat(Infinity);

  return {
    code: `
    ${complierComponentImport(imports.concat(childrenImports))}
  @Component({
      selector: 'eo-${_.kebabCase(id)}',
      template: \`${template}\`
  })
  export class ${id}Component implements OnInit {
    ${complierRegisterData(data)}
    constructor(${complierInject(imports)}) {
      ${complierInitData(data)}
    }
    async ngOnInit(): Promise<void> {
      ${complierInit(init)}
    }
   ${complierMethod(methods)}
  }

  ${
    childrenComponent.length
      ? childrenComponent
          .map((child) => {
            return `
      @Component({
          selector: 'child-${_.kebabCase(child.id)}',
          template: \`${child.template}\`
      })
      export class ${child.id}Component implements OnInit {
        ${complierRegisterData(child.data)}
        constructor(${complierInject(child.imports.flat(Infinity))}) {
          ${complierInitData(child.data)}
        }
        async ngOnInit(): Promise<void> {
          ${complierInit(child.init)}
        }
      ${complierMethod(child.methods)}
      }
    `;
          })
          .join("\n")
      : ""
  }
  `,
    path: `./${_.kebabCase(id)}.component.ts`,
    children: [
      // TODO scss
    ]
  };
};
