import * as _ from "lodash";
import { complierComponent } from "./complierC";

export const complierModule = ({
  id,
  children: _children,
  link,
  components,
  imports
}) => {
  const complierModuleImports = (list) =>
    Object.entries(
      list
        .flat(Infinity)
        .filter((it) => it)
        .reduce((total, { target, from }) => {
          total[from] = (total[from] ? [...total[from], target] : [target])
            .flat(Infinity)
            .filter(
              (it) =>
                it.type &&
                ["module", "component", "service", "base"].includes(it.type)
            );
          return total;
        }, {})
    )
      .map(([from, item]: any) => ({
        from,
        target: Array.from(new Set(item.map((it) => it.name)))
      }))
      .filter(({ from, target }) => target.length)
      .reduce(
        (total, { from, target }) =>
          total + `import {${target.join(",")}} from '${from}'\n`,
        ""
      );

  const moduleList = (list) =>
    Array.from(
      new Set(
        list
          .flat(Infinity)
          .filter((it) => it)
          .reduce((total, { target }) => [...total, target], [])
          .flat(Infinity)
          .filter((it) => it.type === "module")
          .map((it) => it.name)
      )
    );

  const serviceList = (list) =>
    Array.from(
      new Set(
        list
          .flat(Infinity)
          .filter((it) => it)
          .reduce((total, { target }) => [...total, target], [])
          .flat(Infinity)
          .filter((it) => it.type === "service" && it.ignore !== true)
          .map((it) => it.name)
      )
    );

  const componentList = (list) =>
    Array.from(
      new Set(
        list
          .flat(Infinity)
          .filter((it) => it)
          .map((it) => it + "Component")
      )
    );

  const linkComponentList = (list) =>
    Array.from(
      new Set(
        list
          .flat(Infinity)
          .filter((it) => it)
          .reduce((total, { target }) => [...total, target], [])
          .flat(Infinity)
          .filter((it) => it.type === "component")
          .map((it) => it.name)
      )
    );

  const complierImportComponent = (list) =>
    Array.from(new Set(list.flat(Infinity).filter((it) => it))).reduce(
      (total, name: any) =>
        total +
        `import {${_.upperFirst(name)}Component} from './${_.kebabCase(
          name
        )}.component'\n`,
      ""
    );

  const exportList = (list) =>
    Array.from(
      new Set(
        list
          .flat(Infinity)
          .filter((it) => it)
          .reduce((total, { target }) => [...total, target], [])
          .flat(Infinity)
          .filter((it) => it.export)
          .map((it) => it.name)
      )
    );
  const children =
    _children?.map((it) => it.render()).map(complierComponent) || [];
  return {
    code: `
    ${complierModuleImports(imports)}
    import {${id}RoutingModule} from './${_.kebabCase(id)}-routing.module'
    ${complierImportComponent(components)}
    
    @NgModule({
      imports: [${id}RoutingModule,${moduleList(imports)}],
      declarations: [${componentList(components)}, ${linkComponentList(
      imports
    )}],
      exports: [${exportList(imports)}],
      providers: [${serviceList(imports)}],
    })
    export class ${id}Module {}
    `,
    path: `./${_.kebabCase(id)}.module.ts`,
    children
  };
};
