import { elementStruct } from "./types";
import Render from "./render";
import * as _ from "lodash";

export default class Component extends Render {
  id;
  prefix;
  init;
  imports;
  link;
  constructor({ id = "", prefix = "", link = false, imports, children, init }) {
    super({ children });
    if (!id) {
      console.log("Error: Please enter id in Component");
      return;
    }
    this.id = Render.toCamel(id);
    this.prefix = prefix;
    this.init = init;
    this.imports = imports;
    this.link = link ? [{ name: `${Render.toCamel(id)}`, type: "scss" }] : [];
  }
  render(): elementStruct {
    const componentName = `${this.id}`;
    return {
      id: this.id,
      type: "component",
      template: `${this.children.template}`,
      imports: [
        ...this.children.imports,
        {
          target: ["Component", "OnInit"],
          from: "@angular/core",
          ignore: true
        }
      ],
      link: this.link,
      data: [...this.children.data],
      init: [...this.children.init, this.init],
      methods: [...this.children.methods],
      components: [componentName, ...this.children.components]
    };
  }
}
