import * as _ from "lodash";

const random = () => Math.random().toString(36).slice(-6);

const callbackRender = (code) => {
  if (_.isArray(code)) {
    return `{
      ${code.map(callbackRender).join("\n")}
    }`;
  }
  if (_.isFunction(code)) {
    const string = _.toString(code).trim();
    const start = string.indexOf("{");
    const end = string.lastIndexOf("}");
    const func = string.slice(start + 1, end);
    return func;
  }
  return code;
};

export default class Render {
  elementType;
  children;
  eventCb;
  methods;
  status;
  attr;
  constructor({
    children,
    attr = {},
    event = {},
    status = {},
    elementType = ""
  }) {
    this.elementType = elementType;
    this.children = this.renderChild(children);
    const [evCb, evFunc] = this.renderEvent(event);
    const [stCb, stFunc] = this.renderStatus(status);
    this.eventCb = evCb.concat(stCb);
    this.methods = evFunc.concat(stFunc);
    this.attr = this.renderAttr(attr);
  }
  renderAttr(list) {
    return Object.entries(list).map((it) => {
      const [key, value] = it;
      if (value == null) {
        return key;
      }
      return `${key}="${value}"`;
    });
  }
  renderEvent(event) {
    return Object.entries(event).reduce(
      ([cb, func]: any[], [ev, cbCode]: any[]) => {
        const id = random();
        if (_.isArray(cbCode)) {
          cb.push(`(${ev})="${this.elementType || "e"}${id}Callback()"`);
          func.push(
            `async ${this.elementType || "e"}${id}Callback() {
            // * ${ev} event callback
            ${cbCode
              .map((it) => callbackRender(it))
              .filter((it) => it)
              .join("\n")}}`
          );
          return [cb, func];
        }
        // * object { params, callback }
        const { params, callback } = cbCode;
        cb.push(
          `(${ev})="${this.elementType || "e"}${id}Callback(${params.join(
            ","
          )})"`
        );
        func.push(
          `async ${this.elementType || "e"}${id}Callback(${params.join(",")}) {
            // * ${ev} event callback
            ${callback.map((it) => callbackRender(it)).join("\n")}}`
        );
        return [cb, func];
      },
      [[], []]
    );
  }
  renderStatus(status) {
    return Object.entries(status).reduce(
      ([cb, func]: any[], [ev, cbCode]: any[]) => {
        if (cbCode.length === 0) {
          return [cb, func];
        }
        const id = random();
        cb.push(`[${ev}]="${this.elementType || "e"}${id}Status()"`);
        func.push(
          `${this.elementType || "s"}${id}Status() {
            \/\/ * ${ev} status status
            return ${cbCode.map((it) => callbackRender(it)).join("\n")}}`
        );
        return [cb, func];
      },
      [[], []]
    );
  }
  renderChild(children) {
    const childrenList = children?.map((it) => it.render()) || [];
    return {
      template: `${childrenList.map((it) => it.template).join(" ")}`,
      imports: [...childrenList.map((it) => it.imports)],
      data: [...childrenList.map((it) => it.data)],
      init: [...childrenList.map((it) => it.init)],
      components: [...childrenList.map((it) => it.components)],
      methods: [
        ...(this.methods || []),
        ...childrenList.map((it) => it.methods)
      ],
      resetFn: [...childrenList.map((it) => it.resetFn)],
      elementType: [...childrenList.map((it) => it.elementType)]
    };
  }
  static toCamel(id) {
    return _.upperFirst(_.camelCase(id));
  }
  static callbackRender(data) {
    return callbackRender(data);
  }
}
